#include<LINK/stkWrapper/LINK_STk.h>
#include<LINK/stkWrapper/Wrapper.h>
#include<LINK/stkWrapper/Register.h>
#include <LINK/userobj/PostAnalyze.h>


 
static PRIMITIVE ExtractNGInducedGraphWrapper(SCM parms, int len)
{
        SCM num_verts=UNBOUND, prob=UNBOUND, percent=UNBOUND, flag=UNBOUND,
	    deviate=UNBOUND;

	int deviatE = 1; 

        if (len != 5 && len !=4)  
	{
	   Err("usage: extract-ng-induced-graph <num_verts><edge probability><percent checked>", UNDEFINED);
	   Err("<1 or 0>[<deviation>]", UNDEFINED);
	}

	num_verts = CAR(parms);
        if (!INTEGERP(num_verts))
                Err("extract-ng-induced-graph: bad arg 1, integer expected (num_verts)", num_verts);
        int num_vertS = INTEGER(num_verts);


	prob = CAR(CDR(parms));
	if (!FLONUMP(prob))
		Err("extract-ng-induced-graph: bad arg 2, double expected (edge probability)", prob);
	double proB = FLONM(prob);

	
        percent = CAR(CDR(CDR(parms)));
 
        if (!INTEGERP(percent))
                Err("extract-ng-induced-graph: bad arg 3, integer expected (percent)", percent);
        int percenT = INTEGER(percent);
 

        flag = CAR(CDR(CDR(CDR(parms))));

        if (!INTEGERP(flag))
                Err("extract-ng-induced-graph: bad arg 4, integer expected (flag)", flag);
        int flaG = INTEGER(flag);


	if (len == 5)
	{
        	deviate = CAR(CDR(CDR(CDR(CDR(parms)))));

        	if (!INTEGERP(deviate))
                	Err("extract-ng-induced-graph: bad arg 5, integer expected (deviation)",
			     deviate);
        int deviatE = INTEGER(deviate);
	}

        Graph* induced_graph = ExtractNGInducedGraph(num_vertS, proB, percenT,
						     flaG, deviatE);

        Register<Graph>::createEntry(Wrapper<MUBinGraph>::type,
                                     Wrapper<MUBinGraph>::name, induced_graph);
 
	Graph **gp = new Graph*(induced_graph);
        return STk_make_CXXwrapper(Wrapper<MUBinGraph*>::type,
                Wrapper<MUBinGraph*>::name, (void *) gp, LINK_DYNAMIC);
} 
 
void STk_ExtractNGInducedGraph_Init()
{
        STk_add_new_cpp_primitive("extract-ng-induced-graph",       // cmd name
                                  tc_lsubr,            // # parms
                                  (PRIMITIVE (*)(...))ExtractNGInducedGraphWrapper);
}
#include <stk.h>
#include<LINK/stkWrapper/LINK_STk.h>
#include<LINK/stkWrapper/Wrapper.h>
#include<LINK/stkWrapper/Register.h>
#include <LINK/stkWrapper/CollectionWrapper.h>
#include <LINK/userobj/PostAnalyze.h>


 
static PRIMITIVE ExtractWrapper(SCM parms, int len)
{
        SCM g=UNBOUND, percent=UNBOUND, flag=UNBOUND, deviate=UNBOUND;

        Graph *gr;

	int deviatE = 1; 

        if (len != 5 && len !=4)  
	{
	   Err("usage: extract <num_verts><edge probability><percent checked><Sequence>",
		UNDEFINED);
	   Err("<1 or 0>[<deviation>]", UNDEFINED);
	}

	g = CAR(parms);
        if (!(gr = getGraph(g)))   // see src/stkWrapper/stkUtils.cc
                Err("extract: bad arg 1, expected <graph*> ", g);
 

        percent = CAR(CDR(parms));
 
        if (!INTEGERP(percent))
                Err("extract: bad arg 2, integer expected (percent)", percent);
        int percenT = INTEGER(percent);
 

        flag = CAR(CDR(CDR(parms)));

        if (!INTEGERP(flag))
                Err("extract: bad arg 3, integer expected (flag)", flag);
        int flaG = INTEGER(flag);


	SCM vset = CAR(CDR(CDR(CDR(parms))));

        int& vset_type = Wrapper<Sequence<Vertex*> >::type;
        if (!CXX_TYPEP(vset, vset_type))
               Err("digraph: bad arg 4, <sequence<vertex*>> expected", vset);

        Sequence<Vertex*>* vertices = (Sequence<Vertex*>*) CLASSDATA(vset);



	if (len == 5)
	{
        	deviate = CAR(CDR(CDR(CDR(CDR(parms)))));

        	if (!INTEGERP(deviate))
                	Err("extract: bad arg 5, integer expected (deviation)",
			     deviate);
        int deviatE = INTEGER(deviate);
	}

        Graph* induced_graph = Extract(gr, percenT, flaG, *vertices, deviatE);

        Register<Graph>::createEntry(Wrapper<MUBinGraph>::type,
                                     Wrapper<MUBinGraph>::name, induced_graph);
 
	Graph **gp = new Graph*(induced_graph);
        return STk_make_CXXwrapper(Wrapper<MUBinGraph*>::type,
                Wrapper<MUBinGraph*>::name, (void *) gp, LINK_DYNAMIC);
} 
 
void STk_Extract_Init()
{
        STk_add_new_cpp_primitive("extract",       // cmd name
                                  tc_lsubr,            // # parms
                                  (PRIMITIVE (*)(...))ExtractWrapper);
}
#include<LINK/stkWrapper/LINK_STk.h>
#include<LINK/stkWrapper/Wrapper.h>
#include <LINK/userobj/PostAnalyze.h>

 
static PRIMITIVE FindDistributionWrapper(SCM parms, int len)
{
        SCM type, num_verts=UNBOUND, num_graphs=UNBOUND,
		edge_prob=UNBOUND, seed=UNBOUND;
        Graph *gr;
 
        if (len != 5)  Err("find-distribution: 5 arguments expected", UNDEFINED);
 

        num_verts = CAR(parms);
        if (!INTEGERP(num_verts))
          Err("find-distribution: bad arg 1, int expected", num_verts);
	int verts = INTEGER(num_verts);


        edge_prob = CAR(CDR(parms));
        if (!FLONUMP(edge_prob))
                Err("find-distribution: bad arg 2, double expected " // changed
		   "(edge probability)", edge_prob);
        double prob = *(edge_prob->storage_as.flonum.data);


	num_graphs = STk_caddr(parms);
	if (!INTEGERP(num_graphs))
	  Err("find_distribution: bad arg 3, int expected(number of graphs)",
	       num_graphs);
	int graphs = INTEGER(num_graphs);

	type = STk_cadddr(parms);
	if (!STRINGP(type))
	  Err("find-distribution: bad arg 4, string expected(type of graph)",
	       type);
	char* graph_type = STRING(type);

	seed = CAR(CDR(CDR(CDR(CDR(parms)))));
	if (!INTEGERP(seed))
	  Err("find_distribution: bad arg 5, int expected(random seed)",
	       seed);
	int seeD = INTEGER(seed);

        int prev_num_graphs = find_distribution(verts, prob, graphs,
					       graph_type, seeD);
	return STk_makeinteger(prev_num_graphs); 
}

void STk_FindDistribution_Init()
{
       STk_add_new_cpp_primitive("find-distribution",       // cmd name
                                  tc_lsubr,            // # parms
                                  (PRIMITIVE (*)(...))FindDistributionWrapper);
}
 
 

#include<LINK/stkWrapper/LINK_STk.h>
#include<LINK/stkWrapper/Wrapper.h>
#include<LINK/stkWrapper/Register.h>
#include <LINK/userobj/PostAnalyze.h>



static PRIMITIVE GetNumbersWrapper(SCM parms, int len)
{
        SCM num_verts=UNBOUND, prob=UNBOUND, percent=UNBOUND, flag=UNBOUND,
            deviate=UNBOUND, seed=UNBOUND;

        int deviatE = 1;

        if (len != 6)
        {
           Err("usage: extract-ng-induced-graph <num_verts><edge probability><percent checked>", UNDEFINED);
           Err("<flag 1 or 0><1 or 0>[<deviation>]<random_seed>", UNDEFINED);
        }

        num_verts = CAR(parms);
        if (!INTEGERP(num_verts))
                Err("extract-ng-induced-graph: bad arg 1, integer expected (num_verts)", num_verts);
        int num_vertS = INTEGER(num_verts);


        prob = CAR(CDR(parms));
        if (!FLONUMP(prob))
                Err("extract-ng-induced-graph: bad arg 2, double expected (edge probability)", prob);
        double proB = FLONM(prob);


        percent = CAR(CDR(CDR(parms)));

        if (!INTEGERP(percent))
                Err("extract-ng-induced-graph: bad arg 3, integer expected (percent)", percent);
        int percenT = INTEGER(percent);

        flag = CAR(CDR(CDR(CDR(parms))));

        if (!INTEGERP(flag))
                Err("extract-ng-induced-graph: bad arg 4, integer expected (flag)", flag);
        int flaG = INTEGER(flag);


        deviate = CAR(CDR(CDR(CDR(CDR(parms)))));

        if (!INTEGERP(deviate))
                 Err("extract-ng-induced-graph: bad arg 5, integer expected (deviation)",
                             deviate);
        deviatE = INTEGER(deviate);
        

        seed = CAR(CDR(CDR(CDR(CDR(CDR(parms))))));

        if (!INTEGERP(seed))
                 Err("extract-ng-induced-graph: bad arg 6, integer expected (random seed)",
                             seed);
        int Seed = INTEGER(seed);
        

	Graph* g = GetNumbers(num_vertS, proB, percenT,
                                                     flaG, deviatE, Seed);
	Register<Graph>::createEntry(Wrapper<MUBinGraph>::type,
                                     Wrapper<MUBinGraph>::name, g);

	Graph** gp = new Graph*(g);
	return STk_make_CXXwrapper(Wrapper<MUBinGraph*>::type,
                Wrapper<MUBinGraph*>::name, (void *) gp, LINK_DYNAMIC);



}

void STk_GetNumbers_Init()
{
        STk_add_new_cpp_primitive("get-numbers",       // cmd name
                                  tc_lsubr,            // # parms
                                  (PRIMITIVE (*)(...))GetNumbersWrapper);
}

#include <stk.h>
#include<LINK/stkWrapper/LINK_STk.h>
#include<LINK/stkWrapper/Wrapper.h>
#include<LINK/stkWrapper/Register.h>
#include <LINK/stkWrapper/CollectionWrapper.h>
#include <LINK/userobj/PostAnalyze.h>


 
static PRIMITIVE InitPostWrapper(SCM g)
{
        Graph *gr;

        if (!(gr = getGraph(g)))   // see src/stkWrapper/stkUtils.cc
                Err("init-post: bad arg 1, expected <graph*> ", g);

        init_post(gr);
        return UNDEFINED;
} 
 
void STk_InitPost_Init()
{
        STk_add_new_cpp_primitive("init-post",       // cmd name
                                  tc_subr_1,            // # parms
                                  (PRIMITIVE (*)(...))InitPostWrapper);
}
#include<LINK/layout/Layouts.h>
#include<LINK/stkWrapper/LINK_STk.h>
#include<LINK/stkWrapper/Wrapper.h>
#include <LINK/userobj/PostAnalyze.h>


char * getViewName(SCM);

PRIMITIVE PartitioningLayoutWrapper(SCM g)
{
        Graph *gr;
        if (!(gr = getGraph(g)))
          Err("add-subgraph: bad arg 1, <graph*> expected", g);
 
        PartitioningLayout(gr);
 
        return UNDEFINED;
}
static PRIMITIVE PartitioningLayoutAnimCmd(SCM g, SCM gv)
{
        Graph *gr;
        if (!(gr = getGraph(g)))
                Err("partitioning-layout!: bad arg 1, graph expected", g);
        PartitioningLayout(gr);
        char *vname = getViewName(gv);
        path_draw(vname);
        layoutVertices(vname);

        return UNDEFINED;
}

 
//
// This is what must go into userinit.cc
//
void STk_PartitioningLayout_Init()
{
        STk_add_new_cpp_primitive("partitioning-layout", tc_subr_1,
                               (PRIMITIVE (*)(...))PartitioningLayoutWrapper);
        STk_add_new_cpp_primitive("partitioning-layout*", tc_subr_2,
                               (PRIMITIVE (*)(...))PartitioningLayoutAnimCmd);
}

#include <stk.h>
#include<LINK/stkWrapper/LINK_STk.h>
#include<LINK/stkWrapper/Wrapper.h>
#include<LINK/stkWrapper/Register.h>
#include <LINK/stkWrapper/CollectionWrapper.h>
#include <LINK/userobj/PostAnalyze.h>

 
static PRIMITIVE LeftPartitionWrapper(SCM g)
{
        SCM vs, view=UNBOUND, num_steps=UNBOUND;
        Graph *gr;
        if (!(gr = getGraph(g)))   // see src/stkWrapper/stkUtils.cc
                Err("left-partition: expected <graph*> ", g);

        Set<Vertex*> result = PlacedPartitions(gr, LEFT);
        Set<Vertex*>* new_copy = new Set<Vertex*> (result);
 
        return STk_make_CXXwrapper(
                Wrapper<Set<Vertex*>  >::type,     // stkWrapper/Wrapper.h
                Wrapper<Set<Vertex*>  >::name,
                (void *) new_copy,
                LINK_DYNAMIC);          // tells STk's garbage collector
}
 
static PRIMITIVE RightPartitionWrapper(SCM g)
{
        SCM vs, view=UNBOUND, num_steps=UNBOUND;
        Graph *gr;
        if (!(gr = getGraph(g)))   // see src/stkWrapper/stkUtils.cc
                Err("right-partition: expected <graph*> ", g);

        Set<Vertex*> result = PlacedPartitions(gr, RIGHT);
        Set<Vertex*>* new_copy = new Set<Vertex*> (result);
 
        return STk_make_CXXwrapper(
                Wrapper<Set<Vertex*>  >::type,     // stkWrapper/Wrapper.h
                Wrapper<Set<Vertex*>  >::name,
                (void *) new_copy,
                LINK_DYNAMIC);          // tells STk's garbage collector
}
 
static PRIMITIVE UnplacedWrapper(SCM g)
{
        SCM vs, view=UNBOUND, num_steps=UNBOUND;
        Graph *gr;
        if (!(gr = getGraph(g)))   // see src/stkWrapper/stkUtils.cc
                Err("unplaced: expected <graph*> ", g);

        Set<Vertex*> result = PlacedPartitions(gr, UNPLACED);
        Set<Vertex*>* new_copy = new Set<Vertex*> (result);
 
        return STk_make_CXXwrapper(
                Wrapper<Set<Vertex*>  >::type,     // stkWrapper/Wrapper.h
                Wrapper<Set<Vertex*>  >::name,
                (void *) new_copy,
                LINK_DYNAMIC);          // tells STk's garbage collector
}
 
void STk_PlacedPartitions_Init()
{
        STk_add_new_cpp_primitive("right-partition",       // cmd name
                                  tc_subr_1,            // # parms
                                  (PRIMITIVE (*)(...))RightPartitionWrapper);
        STk_add_new_cpp_primitive("left-partition",       // cmd name
                                  tc_subr_1,            // # parms
                                  (PRIMITIVE (*)(...))LeftPartitionWrapper);
        STk_add_new_cpp_primitive("unplaced",       // cmd name
                                  tc_subr_1,            // # parms
                                  (PRIMITIVE (*)(...))UnplacedWrapper);
}
 
 
 
 
#include<LINK/stkWrapper/LINK_STk.h>
#include<LINK/stkWrapper/Wrapper.h>
#include <LINK/userobj/PostAnalyze.h>

 
PRIMITIVE PostAnalyzeWrapper(SCM parms, int len)
{
	SCM vs=UNBOUND, g=UNBOUND, st=UNBOUND; 

	if (len == 0) {
		cerr << "post-anaylze: expected <graph*>, [set<vertex*>*], [int steps]" << endl;
		exit(2);
	}

	int steps = 0;
	Set<Vertex*> s;
	g = CAR(parms);
        Graph *gr;
        if (!(gr = getGraph(g)))   // see src/stkWrapper/stkUtils.cc
                Err("post-analyze: expected <graph*> ", g);

	if (len > 1) {
		vs = CAR(CDR(parms));
		if (INTEGERP(vs)) {
			steps = (INTEGER(vs));
		} else {
	        	int& vset_type = Wrapper<Set<Vertex*> >::type;
	        	if (!CXX_TYPEP(vs, vset_type))
	          		Err("add-subgraph: bad arg 2, <set<vertex*> > expected", vs);
	        	s = *(Set<Vertex*>*) CLASSDATA(vs);
			steps = 0;
			if (len > 2) {
				st = CAR(CDR(CDR(parms)));
				if (!INTEGERP(st))
					steps = 0;
				else    steps = (INTEGER(st));
			}
		}
	}
        Sequence<Vertex*>  result = PostAnalyze(gr, s, steps);
        Sequence<Vertex*> * new_copy = new Sequence<Vertex*> (result);
 
        return STk_make_CXXwrapper(
                Wrapper<Sequence<Vertex*>  >::type,     // stkWrapper/Wrapper.h
                Wrapper<Sequence<Vertex*>  >::name,
                (void *) new_copy,
                LINK_DYNAMIC);          // tells STk's garbage collector
                                        // to delete new_copy when the user
                                        // is finished with it
}
 
//
// This is what must go into userinit.cc
//
void STk_PostAnalyze_Init()
{
        STk_add_new_cpp_primitive("post-analyze",       // cmd name
                                  tc_lsubr,            // # parms
                                  (PRIMITIVE (*)(...))PostAnalyzeWrapper);
}

//genMethods "-t" "Testobj*" "-h" "Array" "-I" "LINK/basic/Array.h" "-I" "LINK/userobj/Testobj.h" "-d" "userobj" 
#include <stk.h>
#include <LINK/stkWrapper/LINK_STk.h>

#include <LINK/basic/Array.h>

#include <LINK/userobj/Testobj.h>

#include <LINK/stkWrapper/Wrapper.h>
#include <LINK/stkWrapper/Register.h>
#include <LINK/stkWrapper/ArrayWrapper.h>


typedef Testobj* _Testobj_;
typedef Array<Testobj* > _Array_Testobj__;

#define _Array_Testobj__DATA(x) ((Array<Testobj* >*) (EXTDATA(x)))
#define _Array_Testobj__IS_DATA(x) (TYPEP(x, tc_CXXwrapper) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Array<Testobj* > >::type))
#define _Array_Testobj__ISNT_DATA(x) (!_Array_Testobj__IS_DATA(x))

#define _Array_Testobj__CXX_DATA(x) (*(Array<Testobj* >*) (EXTDATA(VAL(x))))
#define _Array_Testobj__CXX_IS_DATA(x) (INSTANCEP(x) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Array<Testobj* > >::type))
#define _Array_Testobj__CXX_ISNT_DATA(x) (!_Array_Testobj__CXX_IS_DATA(x))



// define your new methods above 
// and register them using STk_add_new_cpp_primitive() below

void genExtra_Array_Testobj__Methods() {}
//genMethods "-t" "Testobj*" "-h" "Collection" "-I" "LINK/basic/SetFuncs.h" "-I" "LINK/basic/Collection.h" "-I" "LINK/userobj/Testobj.h" "-d" "userobj" 
#include <stk.h>
#include <LINK/stkWrapper/LINK_STk.h>

#include <LINK/basic/SetFuncs.h>

#include <LINK/basic/Collection.h>

#include <LINK/userobj/Testobj.h>

#include <LINK/stkWrapper/Wrapper.h>
#include <LINK/stkWrapper/Register.h>
#include <LINK/stkWrapper/CollectionWrapper.h>


typedef Testobj* _Testobj_;
typedef Collection<Testobj* > _Collection_Testobj__;

#define _Collection_Testobj__DATA(x) ((Collection<Testobj* >*) (EXTDATA(x)))
#define _Collection_Testobj__IS_DATA(x) (TYPEP(x, tc_CXXwrapper) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Collection<Testobj* > >::type))
#define _Collection_Testobj__ISNT_DATA(x) (!_Collection_Testobj__IS_DATA(x))

#define _Collection_Testobj__CXX_DATA(x) (*(Collection<Testobj* >*) (EXTDATA(VAL(x))))
#define _Collection_Testobj__CXX_IS_DATA(x) (INSTANCEP(x) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Collection<Testobj* > >::type))
#define _Collection_Testobj__CXX_ISNT_DATA(x) (!_Collection_Testobj__CXX_IS_DATA(x))



// define your new methods above 
// and register them using STk_add_new_cpp_primitive() below

void genExtra_Collection_Testobj__Methods() {}
//genMethods "-t" "Testobj*" "-h" "Container" "-I" "LINK/basic/Container.h" "-I" "LINK/userobj/Testobj.h" "-d" "userobj" 
#include <stk.h>
#include <LINK/stkWrapper/LINK_STk.h>

#include <LINK/basic/Container.h>

#include <LINK/userobj/Testobj.h>

#include <LINK/stkWrapper/Wrapper.h>
#include <LINK/stkWrapper/Register.h>


typedef Testobj* _Testobj_;
typedef Container<Testobj* > _Container_Testobj__;

#define _Container_Testobj__DATA(x) ((Container<Testobj* >*) (EXTDATA(x)))
#define _Container_Testobj__IS_DATA(x) (TYPEP(x, tc_CXXwrapper) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Container<Testobj* > >::type))
#define _Container_Testobj__ISNT_DATA(x) (!_Container_Testobj__IS_DATA(x))

#define _Container_Testobj__CXX_DATA(x) (*(Container<Testobj* >*) (EXTDATA(VAL(x))))
#define _Container_Testobj__CXX_IS_DATA(x) (INSTANCEP(x) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Container<Testobj* > >::type))
#define _Container_Testobj__CXX_ISNT_DATA(x) (!_Container_Testobj__CXX_IS_DATA(x))



// define your new methods above 
// and register them using STk_add_new_cpp_primitive() below

void genExtra_Container_Testobj__Methods() {}
//genMethods "-t" "Testobj*" "-h" "List" "-I" "LINK/basic/List.h" "-I" "LINK/userobj/Testobj.h" "-d" "userobj" 
#include <stk.h>
#include <LINK/stkWrapper/LINK_STk.h>

#include <LINK/basic/List.h>

#include <LINK/userobj/Testobj.h>

#include <LINK/stkWrapper/Wrapper.h>
#include <LINK/stkWrapper/Register.h>
#include <LINK/stkWrapper/ListWrapper.h>


typedef Testobj* _Testobj_;
typedef List<Testobj* > _List_Testobj__;

#define _List_Testobj__DATA(x) ((List<Testobj* >*) (EXTDATA(x)))
#define _List_Testobj__IS_DATA(x) (TYPEP(x, tc_CXXwrapper) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<List<Testobj* > >::type))
#define _List_Testobj__ISNT_DATA(x) (!_List_Testobj__IS_DATA(x))

#define _List_Testobj__CXX_DATA(x) (*(List<Testobj* >*) (EXTDATA(VAL(x))))
#define _List_Testobj__CXX_IS_DATA(x) (INSTANCEP(x) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<List<Testobj* > >::type))
#define _List_Testobj__CXX_ISNT_DATA(x) (!_List_Testobj__CXX_IS_DATA(x))



// define your new methods above 
// and register them using STk_add_new_cpp_primitive() below

void genExtra_List_Testobj__Methods() {}
//genMethods "-t" "Testobj*" "-h" "MSet" "-r" "Collection<Testobj*>" "-I" "LINK/basic/MSet.h" "-I" "LINK/userobj/Testobj.h" "-d" "userobj" 
#include <stk.h>
#include <LINK/stkWrapper/LINK_STk.h>

#include <LINK/basic/MSet.h>

#include <LINK/userobj/Testobj.h>

#include <LINK/stkWrapper/Wrapper.h>
#include <LINK/stkWrapper/Register.h>
#include <LINK/stkWrapper/CollectionWrapper.h>


typedef Testobj* _Testobj_;
typedef MSet<Testobj* > _MSet_Testobj__;

#define _MSet_Testobj__DATA(x) ((MSet<Testobj* >*) (EXTDATA(x)))
#define _MSet_Testobj__IS_DATA(x) (TYPEP(x, tc_CXXwrapper) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<MSet<Testobj* > >::type))
#define _MSet_Testobj__ISNT_DATA(x) (!_MSet_Testobj__IS_DATA(x))

#define _MSet_Testobj__CXX_DATA(x) (*(MSet<Testobj* >*) (EXTDATA(VAL(x))))
#define _MSet_Testobj__CXX_IS_DATA(x) (INSTANCEP(x) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<MSet<Testobj* > >::type))
#define _MSet_Testobj__CXX_ISNT_DATA(x) (!_MSet_Testobj__CXX_IS_DATA(x))



// define your new methods above 
// and register them using STk_add_new_cpp_primitive() below

void genExtra_MSet_Testobj__Methods() {}
//genMethods "-t" "Testobj*" "-h" "Sequence" "-r" "Collection<Testobj*>" "-I" "LINK/basic/Sequence.h" "-I" "LINK/userobj/Testobj.h" "-d" "userobj" 
#include <stk.h>
#include <LINK/stkWrapper/LINK_STk.h>

#include <LINK/basic/Sequence.h>

#include <LINK/userobj/Testobj.h>

#include <LINK/stkWrapper/Wrapper.h>
#include <LINK/stkWrapper/Register.h>
#include <LINK/stkWrapper/CollectionWrapper.h>


typedef Testobj* _Testobj_;
typedef Sequence<Testobj* > _Sequence_Testobj__;

#define _Sequence_Testobj__DATA(x) ((Sequence<Testobj* >*) (EXTDATA(x)))
#define _Sequence_Testobj__IS_DATA(x) (TYPEP(x, tc_CXXwrapper) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Sequence<Testobj* > >::type))
#define _Sequence_Testobj__ISNT_DATA(x) (!_Sequence_Testobj__IS_DATA(x))

#define _Sequence_Testobj__CXX_DATA(x) (*(Sequence<Testobj* >*) (EXTDATA(VAL(x))))
#define _Sequence_Testobj__CXX_IS_DATA(x) (INSTANCEP(x) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Sequence<Testobj* > >::type))
#define _Sequence_Testobj__CXX_ISNT_DATA(x) (!_Sequence_Testobj__CXX_IS_DATA(x))



// define your new methods above 
// and register them using STk_add_new_cpp_primitive() below

void genExtra_Sequence_Testobj__Methods() {}
//genMethods "-t" "Testobj*" "-h" "Set" "-r" "MSet<Testobj*>" "-I" "LINK/basic/Set.h" "-I" "LINK/userobj/Testobj.h" "-d" "userobj" 
#include <stk.h>
#include <LINK/stkWrapper/LINK_STk.h>

#include <LINK/basic/Set.h>

#include <LINK/userobj/Testobj.h>

#include <LINK/stkWrapper/Wrapper.h>
#include <LINK/stkWrapper/Register.h>
#include <LINK/stkWrapper/CollectionWrapper.h>


typedef Testobj* _Testobj_;
typedef Set<Testobj* > _Set_Testobj__;

#define _Set_Testobj__DATA(x) ((Set<Testobj* >*) (EXTDATA(x)))
#define _Set_Testobj__IS_DATA(x) (TYPEP(x, tc_CXXwrapper) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Set<Testobj* > >::type))
#define _Set_Testobj__ISNT_DATA(x) (!_Set_Testobj__IS_DATA(x))

#define _Set_Testobj__CXX_DATA(x) (*(Set<Testobj* >*) (EXTDATA(VAL(x))))
#define _Set_Testobj__CXX_IS_DATA(x) (INSTANCEP(x) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Set<Testobj* > >::type))
#define _Set_Testobj__CXX_ISNT_DATA(x) (!_Set_Testobj__CXX_IS_DATA(x))



// define your new methods above 
// and register them using STk_add_new_cpp_primitive() below

void genExtra_Set_Testobj__Methods() {}
//genMethods "-t" "Testobj*" "-h" "SortedArray" "-r" "Array<Testobj*>" "-I" "LINK/basic/Array.h" "-I" "LINK/userobj/Testobj.h" "-d" "userobj" 
#include <stk.h>
#include <LINK/stkWrapper/LINK_STk.h>

#include <LINK/basic/SortedArray.h>

#include <LINK/userobj/Testobj.h>

#include <LINK/stkWrapper/Wrapper.h>
#include <LINK/stkWrapper/Register.h>
#include <LINK/stkWrapper/ArrayWrapper.h>


typedef Testobj* _Testobj_;
typedef SortedArray<Testobj* > _SortedArray_Testobj__;

#define _SortedArray_Testobj__DATA(x) ((SortedArray<Testobj* >*) (EXTDATA(x)))
#define _SortedArray_Testobj__IS_DATA(x) (TYPEP(x, tc_CXXwrapper) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<SortedArray<Testobj* > >::type))
#define _SortedArray_Testobj__ISNT_DATA(x) (!_SortedArray_Testobj__IS_DATA(x))

#define _SortedArray_Testobj__CXX_DATA(x) (*(SortedArray<Testobj* >*) (EXTDATA(VAL(x))))
#define _SortedArray_Testobj__CXX_IS_DATA(x) (INSTANCEP(x) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<SortedArray<Testobj* > >::type))
#define _SortedArray_Testobj__CXX_ISNT_DATA(x) (!_SortedArray_Testobj__CXX_IS_DATA(x))



// define your new methods above 
// and register them using STk_add_new_cpp_primitive() below

void genExtra_SortedArray_Testobj__Methods() {}
//genMethods "-t" "Testobj*" "-h" "SortedList" "-r" "List<Testobj*>" "-I" "LINK/basic/List.h" "-I" "LINK/userobj/Testobj.h" "-d" "userobj" 
#include <stk.h>
#include <LINK/stkWrapper/LINK_STk.h>

#include <LINK/basic/SortedList.h>

#include <LINK/userobj/Testobj.h>

#include <LINK/stkWrapper/Wrapper.h>
#include <LINK/stkWrapper/Register.h>
#include <LINK/stkWrapper/ListWrapper.h>


typedef Testobj* _Testobj_;
typedef SortedList<Testobj* > _SortedList_Testobj__;

#define _SortedList_Testobj__DATA(x) ((SortedList<Testobj* >*) (EXTDATA(x)))
#define _SortedList_Testobj__IS_DATA(x) (TYPEP(x, tc_CXXwrapper) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<SortedList<Testobj* > >::type))
#define _SortedList_Testobj__ISNT_DATA(x) (!_SortedList_Testobj__IS_DATA(x))

#define _SortedList_Testobj__CXX_DATA(x) (*(SortedList<Testobj* >*) (EXTDATA(VAL(x))))
#define _SortedList_Testobj__CXX_IS_DATA(x) (INSTANCEP(x) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<SortedList<Testobj* > >::type))
#define _SortedList_Testobj__CXX_ISNT_DATA(x) (!_SortedList_Testobj__CXX_IS_DATA(x))



// define your new methods above 
// and register them using STk_add_new_cpp_primitive() below

void genExtra_SortedList_Testobj__Methods() {}
//genMethods "-t" "Testobj" "-I" "LINK/userobj/Testobj.h" "-d" "userobj" "-y" "display_testobj" "-w" "free_testobj" 
#include <stk.h>
#include <LINK/stkWrapper/LINK_STk.h>
#include <LINK/userobj/Testobj.h>
#include <LINK/stkWrapper/Wrapper.h>
#include <LINK/stkWrapper/Register.h>
void display_testobj(SCM, SCM, int);
void free_testobj(SCM);
typedef Testobj _Testobj;
#define _TestobjDATA(x) ((Testobj*) (EXTDATA(x)))
#define _TestobjIS_DATA(x) (TYPEP(x, tc_CXXwrapper) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Testobj >::type))
#define _TestobjISNT_DATA(x) (!_TestobjIS_DATA(x))
#define _TestobjCXX_DATA(x) (*(Testobj*) (EXTDATA(VAL(x))))
#define _TestobjCXX_IS_DATA(x) (INSTANCEP(x) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Testobj >::type))
#define _TestobjCXX_ISNT_DATA(x) (!_TestobjCXX_IS_DATA(x))
void display_testobj(SCM, SCM, int);
void mark_testobj(SCM);
void free_testobj(SCM);
#include<strstream.h>
void display_testobj(SCM c, SCM port, int mode)
{
        ostrstream oss;
        if (_TestobjDATA(c))
        	oss << *_TestobjDATA(c);
        oss << ends;
        char *buffer = oss.str();
        Puts(buffer, PORT_FILE(port));
        delete buffer;
}
void mark_testobj(SCM p)
{
}
void free_testobj(SCM p)
{
        if (!EXTSTATICP(p)) {
                delete _TestobjDATA(p);
        }
}
// define your new methods above 
// and register them using STk_add_new_cpp_primitive() below
void genExtra_TestobjMethods() {}
//genMethods "-g" "-t" "Testobj*" "-I" "LINK/userobj/Testobj.h" "-d" "userobj" "-y" "display_testobjptr" "-m" "mark_testobjptr" "-w" "free_testobjptr" 
#include <stk.h>
#include <LINK/stkWrapper/LINK_STk.h>
#include <LINK/userobj/Testobj.h>
#include <LINK/stkWrapper/Wrapper.h>
#include <LINK/stkWrapper/Register.h>
void display_testobjptr(SCM, SCM, int);
void mark_testobjptr(SCM);
void free_testobjptr(SCM);
typedef Testobj* _Testobj_;
#define _Testobj_DATA(x) ((Testobj**) (EXTDATA(x)))
#define _Testobj_IS_DATA(x) (TYPEP(x, tc_CXXwrapper) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Testobj* >::type))
#define _Testobj_ISNT_DATA(x) (!_Testobj_IS_DATA(x))
#define _Testobj_CXX_DATA(x) (*(Testobj**) (EXTDATA(VAL(x))))
#define _Testobj_CXX_IS_DATA(x) (INSTANCEP(x) && \
		TYPEP(STk_slot_ref(x,val_scm), tc_CXXwrapper) && \
		(EXTID(STk_slot_ref(x,val_scm)) == Wrapper<Testobj* >::type))
#define _Testobj_CXX_ISNT_DATA(x) (!_Testobj_CXX_IS_DATA(x))
#include<strstream.h>
void display_testobjptr(SCM c, SCM port, int mode)
{
        ostrstream oss;
        if (*_Testobj_DATA(c))
        	oss << **_Testobj_DATA(c);
        oss << ends;
        char *buffer = oss.str();
        Puts(buffer, PORT_FILE(port));
        delete buffer;
}
void mark_testobjptr(SCM p)
{
        //cout << "mark_testobjptr" << endl;
        Register<Testobj>::markEntry(*_Testobj_DATA(p));
}
void free_testobjptr(SCM p)
{
        //cout << "free_testobjptr" << endl;
        if (!EXTSTATICP(p)) {
                delete _Testobj_DATA(p);
        }
}
static PRIMITIVE MakeTestobjCmd(SCM num)
{
        Tcl_HashEntry *p;
        SCM objptr;
        Testobj *ap, **app;
        int not_found;
        if (NINTEGERP(num))
                Err("usage:  make-testobj <integer>", NIL);
        ap = new Testobj(INTEGER(num));
        //Register<Testobj>::createEntry(ap);
        Register<Testobj>::createEntry(Wrapper<Testobj>::type, 
				       Wrapper<Testobj>::name, ap);
        app = new Testobj *(ap);
        return STk_make_CXXwrapper(Wrapper<Testobj*>::type,
                        Wrapper<Testobj*>::name, (void *) app, LINK_DYNAMIC);
}
static PRIMITIVE eqvTestobj(SCM a, SCM b)
{
  if (Wrapper<Testobj*>::compareItems(a, b) == 0)
        return Truth;
  return Ntruth;
}
static PRIMITIVE lthTestobj(SCM a, SCM b)
{
  if (Wrapper<Testobj*>::compareItems(a, b) < 0)
        return Truth;
  return Ntruth;
}
static PRIMITIVE leTestobj(SCM a, SCM b)
{
  if (Wrapper<Testobj*>::compareItems(a, b) <= 0)
        return Truth;
  return Ntruth;
}
static PRIMITIVE grTestobj(SCM a, SCM b)
{
  if (Wrapper<Testobj*>::compareItems(a, b) > 0)
        return Truth;
  return Ntruth;
}
static PRIMITIVE geTestobj(SCM a, SCM b)
{
  if (Wrapper<Testobj*>::compareItems(a, b) >= 0)
        return Truth;
  return Ntruth;
}
static PRIMITIVE GetTestobjSize(SCM a)
{
  SCM z;
  struct stat *info;
  if (_Testobj_CXX_ISNT_DATA(a))
    Err("testobj-size: bad structure ", a);
  NEWCELL(z, tc_integer);
  SET_INTEGER(z, _Testobj_CXX_DATA(a)->current_size);
  return z;
}
static SCM TestobjpCmd(SCM ctr)
{
        return _Testobj_CXX_IS_DATA(ctr) ? Truth : Ntruth;
}
// define methods above, register below
void genExtra_Testobj_Methods() 
{
        STk_add_new_cpp_primitive("make-testobj", tc_subr_1,
                                        (PRIMITIVE (*)(...))MakeTestobjCmd);
        STk_add_new_cpp_primitive("testobj-eqv?", tc_subr_2,
                                        (PRIMITIVE (*)(...))eqvTestobj);
        STk_add_new_cpp_primitive("testobj-<", tc_subr_2,
                                        (PRIMITIVE (*)(...))lthTestobj);
        STk_add_new_cpp_primitive("testobj-<=", tc_subr_2,
                                        (PRIMITIVE (*)(...))leTestobj);
        STk_add_new_cpp_primitive("testobj->", tc_subr_2,
                                        (PRIMITIVE (*)(...))grTestobj);
        STk_add_new_cpp_primitive("testobj->=", tc_subr_2,
                                        (PRIMITIVE (*)(...))geTestobj);
        STk_add_new_cpp_primitive("testobj?", tc_subr_1,
                                        (PRIMITIVE (*)(...))TestobjpCmd);
        STk_add_new_cpp_primitive("size-testobj", tc_subr_1,
                                        (PRIMITIVE (*)(...))GetTestobjSize);
        addTwoArgMethod("eqv?", Wrapper<Testobj*>::name,
                        Wrapper<Testobj*>::name, "testobj-eqv?", LINK_OVL);
        addBooleanMethod("<",
                        Wrapper<Testobj*>::name, "testobj-<", LINK_OVL);
        addBooleanMethod("<=",
                        Wrapper<Testobj*>::name, "testobj-<=", LINK_OVL);
        addBooleanMethod(">",
                        Wrapper<Testobj*>::name, "testobj->", LINK_OVL);
        addBooleanMethod(">=",
                        Wrapper<Testobj*>::name, "testobj->=", LINK_OVL);
}
